#include "adc.h"
#include <stdint.h>
#include <avr/io.h>

/*
Use the analog/digital-converter a more comfortable way.

Missing features: input-gain, auto trigger enable

Target:      ATMega48, ATMega88, ATMega168, ATMega328
Last Change: 2012-03-27 (Frank Baettermann)
*/

void adc_init(adc_voltageReference voltageReference, adc_adjustResult adjustResult, adc_interrupt interrupt, adc_autoTrigger autoTrigger, adc_autoTriggerSource autoTriggerSource)
{
    // ADMUX – ADC Multiplexer Selection Register
    ADMUX = voltageReference | adjustResult;
    // ADCSRA – ADC Control and Status Register A
    ADCSRA = autoTrigger | interrupt | ADC_PRESCALER_REG;
    // ADCSRB – ADC Control and Status Register B
    ADCSRB = autoTriggerSource;
}


void adc_setChannel(uint8_t channel)
{
    // Map
    // -----------------------------------------
    // 0-7:     singel ended channels
    // 8:       internal temperature reference
    // 9-13:    reserved
    // 14:      internal voltage reference
    // 15:      gnd
    
    // As the MUX3:0-bits are right aligned in ADMUX-register we only mask them
    ADMUX = (ADMUX & 0xf0) | (channel & 0x0f);
}


void adc_enable(void)
{
    ADCSRA |= (1 << ADEN);
}

void adc_disable(void)
{
    ADCSRA &= ~(1 << ADEN);
}

void adc_startConversion(void)
{
    ADCSRA |= (1 << ADSC);
}

// returns value >0 when finished
uint8_t adc_conversionFinished(void)
{
    return ADCSRA & (1 << ADSC);
}

uint16_t adc_readDataRegister(void)
{
    return ADCW;
}

// Convenience function (call adc_init, adc_enable, adc_setChannel first!)
uint16_t adc_singleConversion(void)
{
    adc_startConversion();
    // loop until the conversion is complete
    while (ADCSRA & (1 << ADSC));
    // return result
    return ADCW;
}

